@@ -51,8 +51,8 @@ Rcgminu <- function(par, fn, gr, control = list(), ...) {
     #  Date:  April 2, 2009; revised July 28, 2009
     #################################################################
     # control defaults -- idea from spg
-    ctrl <- list(maxit = 500, maximize = FALSE, trace = 0, eps = 1e-07, 
-        dowarn = TRUE, tol=0)
+    ctrl <- list(maxit = 500, maxfeval=10000, maximize = FALSE, trace = 0, 
+        eps = 1e-07, dowarn = TRUE, tol=0)
     namc <- names(control)
     if (!all(namc %in% names(ctrl))) 
         stop("unknown names in control: ", namc[!(namc %in% names(ctrl))])
@@ -63,6 +63,7 @@ Rcgminu <- function(par, fn, gr, control = list(), ...) {
     else tol<-ctrl$tol
     maxit <- ctrl$maxit  # limit on function evaluations
     maximize <- ctrl$maximize  # TRUE to maximize the function
+    maxfeval <- ctrl$maxfeval
     trace <- ctrl$trace  # 0 for no output, >0 for output (bigger => more output)
     if (trace > 2) 
         cat("trace = ", trace, "\n")
@@ -96,7 +97,7 @@ Rcgminu <- function(par, fn, gr, control = list(), ...) {
     }
     bvec <- par  # copy the parameter vector
     n <- length(bvec)  # number of elements in par vector
-    maxfeval <- round(sqrt(n + 1) * maxit)  # change 091219
+    if (is.null(maxfeval)) maxfeval <- round(sqrt(n + 1) * maxit)  # change 091219
     ig <- 0  # count gradient evaluations
     ifn <- 1  # count function evaluations (we always make 1 try below)
     stepredn <- 0.15  # Step reduction in line search
@@ -325,7 +326,7 @@ Rcgminu <- function(par, fn, gr, control = list(), ...) {
                     msg <- " Converged -- no progress on new CG cycle"
                     if (trace > 0) 
                       cat("\n", msg, "\n")
-                    keekpgoing <- FALSE
+                    keepgoing <- FALSE
                     break  #!!
                   }
                 }  # end else
