@@ -1,7 +1,7 @@
 ############### grnd.R ####################
 grnd<-function(par, userfn, ...) { # using grad from numDeriv
 # with Richardson method
-   tryg<-grad(userfn, par, ...)
+   tryg<-numDeriv::grad(userfn, par, ...)
 }
 ############### end grnd ####################
 
